/////////////////////////////////////////////////////////////////////////////
// Name:        cube.cpp
// Purpose:     wxGLCanvas demo program
// Author:      Julian Smart
// Modified by:
// Created:     04/01/98
// RCS-ID:      $Id: cube.cpp,v 1.9 2002/08/31 22:30:50 GD Exp $
// Copyright:   (c) Julian Smart
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation
#pragma interface
#endif

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/log.h"

#if !wxUSE_GLCANVAS
#error Please set wxUSE_GLCANVAS to 1 in setup.h.
#endif

#include "nviz3d.h"

static GLfloat xrot = 0;
static GLfloat yrot = 0;
static GLfloat zoom = 0;
static float last_x = 0,
             last_y = 0;

//-----------------------------------------------------------------------------
//  NViz3DCanvas class Implementation
//-----------------------------------------------------------------------------

BEGIN_EVENT_TABLE(Nviz3DCanvas, wxGLCanvas)
    EVT_SIZE(Nviz3DCanvas::OnSize)
    EVT_PAINT(Nviz3DCanvas::OnPaint)
    EVT_ERASE_BACKGROUND(Nviz3DCanvas::OnEraseBackground)
    EVT_ENTER_WINDOW( Nviz3DCanvas::OnEnterWindow )
    EVT_MOUSE_EVENTS( Nviz3DCanvas::OnMouse )
END_EVENT_TABLE()

//----------Nviz3DCanvas Constructor-------------------------------------------
Nviz3DCanvas::Nviz3DCanvas(wxWindow *parent, wxWindowID id,
                           const wxPoint& pos, const wxSize& size,
                           long style, const wxString& name):
             wxGLCanvas(parent, (wxGLCanvas*) NULL, id, pos, size, style, name )
{
    m_init = FALSE;
}

//----------Nviz3DCanvas Destructor--------------------------------------------
Nviz3DCanvas::~Nviz3DCanvas()
{
    // Nothing Here
}

//----------Nviz3DCanvas Render Method-----------------------------------------
void Nviz3DCanvas::Render()
{
    // This is apperantly a dummy to avoid an endless succession of paint
    // messages. OnPaint handlers must always create a wxPaintDC.
    wxPaintDC dc(this);

#ifndef __WXMOTIF__
    if (!GetContext()) return;
#endif

    SetCurrent();
    // init OpenGL once, but after SetCurrent
    if (!m_init)
    {
        InitGL();
        m_init = TRUE;
    }

    // clear color and depth buffers
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glPushMatrix();
    glMatrixMode(GL_MODELVIEW);
    glRotatef(yrot, 0.0, 1.0, 0.0);
    glRotatef(xrot, 1.0, 0.0, 0.0);
    glMatrixMode(GL_PROJECTION);
    glTranslatef(0.0, 0.0, zoom);

    GLUquadricObj *gl_neuron = gluNewQuadric();
    gluSphere(gl_neuron, 1.0, 10, 10);

    glPopMatrix();

  glFlush();
  SwapBuffers();
}

//----------Nviz3DCanvas OnEnterWindow Method----------------------------------
void Nviz3DCanvas::OnEnterWindow( wxMouseEvent& event )
{
    SetFocus();
}

//----------Nviz3DCanvas OnPaint Method----------------------------------------
void Nviz3DCanvas::OnPaint( wxPaintEvent& event )
{
    Render();
}

//----------Nviz3DCanvas OnSize Method-----------------------------------------
void Nviz3DCanvas::OnSize(wxSizeEvent& event)
{
    // this is also necessary to update the context on some platforms
    wxGLCanvas::OnSize(event);

    // set GL viewport (not called by wxGLCanvas::OnSize on all platforms...)
    int w, h;
    GetClientSize(&w, &h);
#ifndef __WXMOTIF__
    if (GetContext())
#endif
    {
        SetCurrent();
        glViewport(0, 0, (GLint) w, (GLint) h);
    }
}

//----------Nviz3DCanvas OnMouse Method----------------------------------------
void Nviz3DCanvas::OnMouse(wxMouseEvent &event)
{
// I think all this dragging stuff is a way to waste a cycle to get a last_x or
// y before drawing. Still not working but good enough for now.
    static int dragging = 0;

    if (event.LeftIsDown())
    {
        if (!dragging)
        {
            dragging = 1;
        }
        else
        {
            yrot += (event.GetX() - last_x)*1.0;
            xrot += (event.GetY() - last_y)*1.0;
            Refresh(FALSE);
        }
        last_x = event.GetX();
        last_y = event.GetY();
    }


    if (event.RightIsDown())
    {
        if (!dragging)
        {
            dragging = 0;
        }
        else
        {
            xrot = 0.0;  // If this is not done it will rotate when zooming
            yrot = 0.0;
            zoom  += (event.GetY() - last_y)*0.25;
            Refresh(FALSE);
        }
        last_y = event.GetY();
    }
//    else
//        dragging = 0;
}

void Nviz3DCanvas::DrawNeuron(GLfloat x, GLfloat y, GLfloat z,)
{
}

//----------Nviz3DCanvas OnEraseBackGround Method------------------------------
void Nviz3DCanvas::OnEraseBackground(wxEraseEvent& event)
{
  // Do nothing, to avoid flashing.
}

//----------Nviz3DCanvas InitGL Method-----------------------------------------
void Nviz3DCanvas::InitGL()
{
    SetCurrent();

    // set viewing projection
    glMatrixMode(GL_PROJECTION);
    glFrustum(-1.0F, 1.0F, -1.0F, 1.0F, 1.0F, 100.0F);

    // position viewer
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslated(0.0, 0.0, -4.0);

    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
}

//-----------------------------------------------------------------------------
//  NVizFrame class Implementation
//-----------------------------------------------------------------------------

BEGIN_EVENT_TABLE(NvizFrame, wxFrame)
    EVT_MENU(wxID_EXIT, NvizFrame::OnExit)
END_EVENT_TABLE()

//----------NvizFrame Constructor----------------------------------------------
NvizFrame::NvizFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
                     const wxSize& size, long style)
                     : wxFrame(frame, -1, title, pos, size, style)
{
    glc_nvizcanvas = NULL;
}

//----------NvizFrame OnExit Method----------------------------------------------
void NvizFrame::OnExit(wxCommandEvent &event)
{
    Destroy();
}

//-----------------------------------------------------------------------------
//  NViz3DApp class Implementation
//-----------------------------------------------------------------------------

IMPLEMENT_APP(Nviz3DApp)

//----------Nviz3DApp OnInit Main Method----------------------------------------------
bool Nviz3DApp::OnInit(void)
{
  wxLog::SetTraceMask(wxTraceMessages);

  // Create the main frame window
  NvizFrame *frame = new NvizFrame(NULL, "Cube OpenGL Demo", wxPoint(50, 50),
                               wxSize(400, 300));
  // Give it an icon
#ifdef wx_msw
  frame->SetIcon(wxIcon("mondrian"));
#endif

  // Make a menubar
  wxMenu *winMenu = new wxMenu;

  winMenu->Append(wxID_EXIT, "&Close");
  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(winMenu, "&Window");

  frame->SetMenuBar(menuBar);

  frame->glc_nvizcanvas = new Nviz3DCanvas(frame, -1, wxDefaultPosition, wxDefaultSize);

  // Show the frame
  frame->Show(TRUE);

  return TRUE;
}
